% Copyright 2010 Jakob Heyman
% This code is free software: you can redistribute it and/or modify it under the terms of the GNU
% General Public License; version 3 of the License or any later version. There is no warranty for
% this code. For a copy of the GNU Genaral Public License, see <http://www.gnu.org/licenses/>.

% Boulder exposure age script used in Heyman et al. (2010):
% Too young or too old: evaluating cosmogenic exposure dating based on an analysis of compiled
% boulder exposure ages. Earth and Planetary Science Letters.

% Monte Carlo simulation of multiple-boulder group (random deglaciation age) apparent exposure ages
% assuming prior exposure with a boulder group random duration of prior exposure and individual 
% boulder random prior sample depth. Be-10 surface prod rate due to spallation is
% taken as 98.8% of the total surface prod rate and Be-10 surface prod rate due to muon interaction
% is taken as 1.2% of the total surface prod rate. Depth-dependent spallogenic Be-10 production is
% based on Lal (1991), and muogenic Be-10 production is based on Granger and Smith (2000).

% Contact info: jakob.heyman@natgeo.su.se

clear all;

boulders = [2,3,4,5,6,7,8,9,10,11,12,14];		% nr of boulders per discrete glacial deposit
samplnr = [84,90,59,51,26,10,10,4,2,4,1,1];		% nr of boulder groups with [2,3,4 ... 14] boulders
samplnr = samplnr .* 20;						% nr of Monte Carlo iterations
maxpriordur = 150000;	% maximum duration of prior exposure (yr)
max_pri_d = 500;		% maximum prior sample depth (cm)
maxage = 250000;		% maximum deglaciation age (yr)
rdens = 2.7;			% rock density (g/cm^3)
atten = 160;			% spallogenic prod. effective attenuation length (g/cm^2)
rdc = -log(0.5)/1387000;% radioactive decay constant (yr^-1) (Chmeleff et al. 2010; Korchinek et al. 2010)
spfract = 0.988;		% spallogenic fraction of total surface production rate
mufract = 0.012;		% muogenic fraction of total surface production rate
m1 = 0.76;				% muogenic prod. approxim. coefficient
m2 = 0.11;				% muogenic prod. approxim. coefficient
m3 = 0.13;				% muogenic prod. approxim. coefficient
L1 = 738.6;				% muogenic prod. approxim. effect. atten. length (g/cm^2)
L2 = 2688;				% muogenic prod. approxim. effect. atten. length (g/cm^2)
L3 = 4360;				% muogenic prod. approxim. effect. atten. length (g/cm^2)
ages = [];


for i = 1:numel(boulders)		% loop for boulder group data
	deglacage = rand (1,samplnr (i)) .* maxage;						% random deglaciation-age for boulder group
	priordur = rand (1,samplnr (i)) .* maxpriordur;					% random duration of prior exposure

	for j = 1:boulders(i)		% loop for individual boulder data
		pri_d = rand (1,samplnr (i)) .* max_pri_d;	% random prior sample depth - below: prior exp Be-10 conc
		beconc = (spfract .* exp(-rdens/atten .* pri_d) + mufract .* (m1 .* exp(-rdens/L1 .* pri_d) + ...
		m2 .* exp(-rdens/L2 .* pri_d) + m3 .* exp(-rdens/L3 .* pri_d))) ./ rdc .* (1 - exp(-rdc .* priordur));
		expage (j,:) = log (1 - beconc .* rdc) ./ -rdc + deglacage;		% apparent exposure age
    end
	
	minbould = min (expage);		% pick out boulder group min age
	maxbould = max (expage);		% pick out boulder group max age
	standav = std (expage);			% pick out boulder group standard deviation
	
	% fill ages matrix with group min and max exposure age and individual boulder exposure age
	rowages = rows (ages);
	colexpage = columns (expage);
	for k = 1:(boulders(i))
		ages (rowages + 1:rowages + colexpage, 1) = minbould';		% min age (yr) in column 1
		ages (rowages + 1:rowages + colexpage, 2) = maxbould';		% max age (yr) in column 2
		ages (rowages + 1:rowages + colexpage, 3) = expage (k,:)';	% individual age (yr) in column 3
		rowages = rows (ages);
	end
	
	% fill groupdata matrix with group min and max exposure age and exposure age standard deviation
	groupdata (sum (samplnr (1:i)) - samplnr (1,i) +1:sum (samplnr (1:i)), 1) = minbould';	% min age (yr) in column 1
	groupdata (sum (samplnr (1:i)) - samplnr (1,i) +1:sum (samplnr (1:i)), 2) = maxbould';	% max age (yr) in column 2
	groupdata (sum (samplnr (1:i)) - samplnr (1,i) +1:sum (samplnr (1:i)), 3) = standav';	% stdev (yr) in column 3

	clear expage;
	clear deglacage;
	clear priordur;
	clear pri_d;
	clear minbould;
	clear maxbould;
	clear standav;
end

save prior_group groupdata -ascii;		% save groupdata as ascii file
save prior_ages ages -ascii;			% save ages as ascii file

beep;